help <- 
VRhelp <- function(name = "Contents", pager = options()$pager, 
		     module = NULL, library = NULL)
{
# based on help function in S-PLUS 3.3 (C) 1995 MathSoft Inc.
# modified function by B.D. Ripley 11/96
# for use only with a licensed copy of S-PLUS 3.x
#
# corrects behaviour for libraries with .hlp files.
#
  if(!missing(name) && is.name(sname <- substitute(name)) 
     && !(exists(as.character(sname)) && is.character(name) 
     && length(name) == 1))
    name <- substitute(name)		# name is not yet evaluated
  if(is.language(name) && !is.name(name)) name <- eval(name)
  name <- as.character(name)
  if(length(name) != 1) {
    warning("Can only get help on one item at a time")
    name <- name[1]
  }
  if(name == "") name <- "Contents"
  if(!is.null(module)) {
    if (mode(module) != "character")
      module <- deparse(substitute(module))
    defaultname <- paste(getenv("SHOME"), "\\module\\", module, 
			 "\\", module, ".hlp", sep = "")
  }
  else if(!is.null(library)) {
# We don't assume the library is attached, so search for it
    if (mode(library) != "character")
      library <- deparse(substitute(library))
    if(exists("lib.loc", where = 0)) lib.loc <- get("lib.loc", where = 0)
    else lib.loc <- NULL
    def.lib.loc <- paste(getenv("SHOME"), "library", sep = "\\")
    lib.loc <- c(lib.loc, def.lib.loc)	
    posfiles <- paste(lib.loc, library, 
		      paste(library, "hlp", sep="."), sep = "\\")
    defaultname <- NULL
    for(name in posfiles) {
      if(access(name, 0) == 0) {
	defaultname <- name
	break
      }
    }
    if(is.null(defaultname)) 
      stop(paste("No help file for library",library,"could be found"))
#    defaultname <- paste(getenv("SHOME"), "\\library\\", library, 
#			 "\\", library, ".hlp", sep = "")
  }
  else defaultname <- paste(getenv("SHOME"), "\\cmd\\splus.hlp", sep = "")
  helpname <- VRfind.doc(name, defaultname)[1]
  if(nchar(helpname) == 0) 
    cat("No documentation available for", name, "\n")
  else if(substring(helpname, nchar(helpname)-3) == ".hlp")
    VRhelp.win.file(helpname, name)
  else VRhelp.ascii.file(helpname)
  invisible()
}

VRhelp.win.file <- function(pathname, name)
{
    .C("S_call_for_help",
       as.integer(0),
       as.character(pathname),
       as.character(name))
}

VRhelp.ascii.file <- function(helpname, pager = options()$pager)
{
# add a . to end of filename if no extension:
  helpname <- helpname[1]
  nc <- nchar(helpname)
  if(match(AsciiToInt("."), AsciiToInt(helpname)[(nc - 3):nc], nomatch = 0) == 0)
    helpname <- paste(helpname, ".", sep = "")
  win3(paste(pager, helpname), trans = T, multi = T)
}

VRfind.doc <-
function(what, defaultname=paste(getenv("SHOME"), "\\cmd\\splus.hlp", sep=""))
{
  if(!is.name(what) && !is.character(what) || length(what) != 1)
    stop("what argument must be a character vector of length 1\n")
  found.doc <- character(0)
  sname <- what
  nc <- nchar(what)
# Look up "name.object" as "name" in the databases
  if(substring(what, nc-6) == ".object") sname <- substring(what, 1, nc-7)
  attached <- search()
  m1 <- getenv("SHOME")
  m2 <- paste(m1, "\\library", sep = '')
  m3 <- paste(m1, "\\module", sep = "")

  for(i in seq(along=attached)) {
    if(!exists(sname, where=i)) next
    db <- attached[i]
    nc <- nchar(db)
    if(pmatch(m1, db, nomatch = 0) && !pmatch(m2, db, nomatch = 0) 
       && !pmatch(m3, db, nomatch = 0))
    {
# A system database
      found.doc <- c(found.doc, 
		     paste(getenv("SHOME"), "\\cmd\\splus.hlp", sep=""))
      next
    }
    ending <- substring(db, nc-5, nc)
    if(ending == "\\_Data" || ending == "\\_DATA" || ending == "\\_data") {
      basepath <- substring(db, 1, nc-6)
# emulate an rindex call.
      ncp1 <- nchar(basepath) + 1
      Abasepath <- rev(AsciiToInt(basepath))
      tt <- pmatch(AsciiToInt("\\"), Abasepath, nomatch = ncp1)
      tt <- c(tt, pmatch(AsciiToInt("/"), Abasepath, nomatch = ncp1))
      libname <- substring(basepath, 1 + max(ncp1 - tt))
      if(libname=="trellis" || libname=="Matrix" || libname=="cluster") {
# special case, a library with help in the system help file
	found.doc <- c(found.doc, 
		       paste(getenv("SHOME"), "\\cmd\\splus.hlp", sep=""))
	next
      }
      helpfile <- paste(basepath, "\\", libname, ".hlp", sep="")
      if(access(helpfile, 0) == 0) {
	found.doc <- c(found.doc, helpfile)
	next
      }
    }
    if(access(paste(db, "\\_Help", sep = ""), 0) != 0) next
    truename <- true.file.name(as.character(what), db, write = F)
    helpname <- paste(db, "\\_Help\\", truename, sep = "")
    if(truename != "" && access(helpname, 0) == 0)
      found.doc <- c(found.doc, helpname)
  }
  if(access(defaultname, 0) == 0) found.doc <- c(found.doc, defaultname)
  found.doc
}
